@@ -71,6 +71,15 @@ protected:
 #endif
 	}
 	
+	/// initializes head and tail pointers for empty queue
+	inline void initialize(void) {
+		// initialize with a dummy node since m_head_ptr is always 
+		// pointing to the item before the head of the list
+		m_head_ptr = m_tail_ptr = createNode();
+		m_head_ptr->next = NULL;
+		m_head_ptr->version = 0;
+	}
+	
 	/**
 	 * dequeues the next item from the top of the queue
 	 *
@@ -114,8 +123,23 @@ public:
 	class ConsumerThread {
 	public:
 
-		/// constructor assumes thread is active/running
-		ConsumerThread(void) : m_is_running(true), m_next_ptr(NULL) {}
+		/**
+		 * default constructor used to disable idle wakeup timer
+		 * (assumes thread is active/running)
+		 */
+		ConsumerThread(void) : m_is_running(true), m_next_ptr(NULL),
+			m_wakeup_time(boost::posix_time::not_a_date_time) {}
+
+		/**
+		 * constructor used to enable an idle wakeup timer for the thread
+		 * (assumes thread is active/running)
+		 *
+		 * @param d inactivity wakeup timer duration
+		 */
+		template <typename DurationType>
+		ConsumerThread(const DurationType& d)
+			: m_is_running(true), m_next_ptr(NULL), m_wakeup_time(d)
+		{}
 
 		/// returns true while the consumer thread is active/running
 		inline bool isRunning(void) const { return m_is_running; }
@@ -125,6 +149,14 @@ public:
 
 		/// stops the thread -- if waiting on pop() will return immediately
 		inline void reset(void) { m_is_running = true; m_next_ptr = NULL; }
+		
+		/// returns true if an inactivity wakeup timer is set for the thread
+		inline bool hasWakeupTimer(void) const { return !m_wakeup_time.is_not_a_date_time(); }
+		
+		/// returns absolute wakeup time based on current time
+		inline const boost::posix_time::time_duration& getWakeupTimer(void) const {
+			return m_wakeup_time;
+		}
 
 	private:
 
@@ -132,8 +164,9 @@ public:
 		friend class PionLockedQueue;
 
 		volatile bool		m_is_running;		//< true while the thread is running/active
-		boost::condition	m_wakeup_event;	//< triggered when a new item is available
 		ConsumerThread *	m_next_ptr;		//< pointer to the next idle thread
+		boost::condition	m_wakeup_event;	//< triggered when a new item is available
+		boost::posix_time::time_duration	m_wakeup_time;	//< inactivity wakeup timer duration
 	};
 
 
@@ -142,11 +175,7 @@ public:
 		: m_head_ptr(NULL), m_tail_ptr(NULL), m_idle_ptr(NULL),
 		m_next_version(1), m_size(0)
 	{
-		// initialize with a dummy node since m_head_ptr is always 
-		// pointing to the item before the head of the list
-		m_head_ptr = m_tail_ptr = createNode();
-		m_head_ptr->next = NULL;
-		m_head_ptr->version = 0;
+		initialize();
 	}
 	
 	/// virtual destructor
@@ -167,13 +196,15 @@ public:
 	void clear(void) {
 		boost::mutex::scoped_lock tail_lock(m_tail_mutex);
 		boost::mutex::scoped_lock head_lock(m_head_mutex);
-		while (m_head_ptr->next) {
+		// also delete dummy node and reinitialize it to clear old value
+		while (m_head_ptr) {
 			m_tail_ptr = m_head_ptr;
 			m_head_ptr = m_head_ptr->next;
 			destroyNode(m_tail_ptr);
-			--m_size;
+			if (m_head_ptr)
+				--m_size;
 		}
-		m_tail_ptr = m_head_ptr;
+		initialize();
 	}
 
 	/**
@@ -240,8 +271,16 @@ public:
 				// still empty after acquiring lock
 				thread_info.m_next_ptr = m_idle_ptr;
 				m_idle_ptr = & thread_info;
-				// wait for an item to become available
-				thread_info.m_wakeup_event.wait(tail_lock);
+				// get wakeup time (if any)
+				if (thread_info.hasWakeupTimer()) {
+					// wait for an item to become available
+					const boost::posix_time::ptime wakeup_time(boost::get_system_time() + thread_info.getWakeupTimer());
+					if (!thread_info.m_wakeup_event.timed_wait(tail_lock, wakeup_time))
+						return false;	// timer expired if timed_wait() returns false
+				} else {
+					// wait for an item to become available
+					thread_info.m_wakeup_event.wait(tail_lock);
+				}
 			}
 		}
 		return false;
