@@ -13,6 +13,8 @@
 #include <string>
 #include <boost/noncopyable.hpp>
 #include <boost/logic/tribool.hpp>
+#include <boost/system/error_code.hpp>
+#include <boost/thread/once.hpp>
 #include <pion/PionConfig.hpp>
 #include <pion/PionLogger.hpp>
 #include <pion/net/HTTPMessage.hpp>
@@ -37,6 +39,77 @@ public:
 	/// maximum length for HTTP payload content
 	static const std::size_t		DEFAULT_CONTENT_MAX;
 
+	/// class-specific error code values
+	enum ErrorValue {
+		ERROR_METHOD_CHAR = 1,
+		ERROR_METHOD_SIZE,
+		ERROR_URI_CHAR,
+		ERROR_URI_SIZE,
+		ERROR_QUERY_CHAR,
+		ERROR_QUERY_SIZE,
+		ERROR_VERSION_EMPTY,
+		ERROR_VERSION_CHAR,
+		ERROR_STATUS_EMPTY,
+		ERROR_STATUS_CHAR,
+		ERROR_HEADER_CHAR,
+		ERROR_HEADER_NAME_SIZE,
+		ERROR_HEADER_VALUE_SIZE,
+		ERROR_INVALID_CONTENT_LENGTH,
+		ERROR_CHUNK_CHAR,
+		ERROR_MISSING_CHUNK_DATA,
+		ERROR_MISSING_HEADER_DATA,
+		ERROR_MISSING_TOO_MUCH_CONTENT,
+	};
+	
+	/// class-specific error category
+	class ErrorCategory
+		: public boost::system::error_category
+	{
+	public:
+		const char *name() const { return "HTTPParser"; }
+		std::string message(int ev) const {
+			switch (ev) {
+			case ERROR_METHOD_CHAR:
+				return "invalid method character";
+			case ERROR_METHOD_SIZE:
+				return "method exceeds maximum size";
+			case ERROR_URI_CHAR:
+				return "invalid URI character";
+			case ERROR_URI_SIZE:
+				return "method exceeds maximum size";
+			case ERROR_QUERY_CHAR:
+				return "invalid query string character";
+			case ERROR_QUERY_SIZE:
+				return "query string exceeds maximum size";
+			case ERROR_VERSION_EMPTY:
+				return "HTTP version undefined";
+			case ERROR_VERSION_CHAR:
+				return "invalid version character";
+			case ERROR_STATUS_EMPTY:
+				return "HTTP status undefined";
+			case ERROR_STATUS_CHAR:
+				return "invalid status character";
+			case ERROR_HEADER_CHAR:
+				return "invalid header character";
+			case ERROR_HEADER_NAME_SIZE:
+				return "header name exceeds maximum size";
+			case ERROR_HEADER_VALUE_SIZE:
+				return "header value exceeds maximum size";
+			case ERROR_INVALID_CONTENT_LENGTH:
+				return "invalid Content-Length header";
+			case ERROR_CHUNK_CHAR:
+				return "invalid chunk character";
+			case ERROR_MISSING_HEADER_DATA:
+				return "missing header data";
+			case ERROR_MISSING_CHUNK_DATA:
+				return "missing chunk data";
+			case ERROR_MISSING_TOO_MUCH_CONTENT:
+				return "missing too much content";
+			}
+			return "HTTPParser error";
+		}
+	};
+
 	/**
 	 * creates new HTTPParser objects
 	 *
@@ -51,7 +124,8 @@ public:
 		m_chunked_content_parse_state(PARSE_CHUNK_SIZE_START), m_status_code(0),
 		m_bytes_content_remaining(0), m_bytes_content_read(0),
 		m_bytes_last_read(0), m_bytes_total_read(0),
-		m_max_content_length(max_content_length), m_save_raw_headers(false)
+		m_max_content_length(max_content_length),
+		m_parse_headers_only(false), m_save_raw_headers(false)
 	{}
 
 	/// default destructor
@@ -61,26 +135,29 @@ public:
 	 * parses an HTTP message including all payload content it might contain
 	 *
 	 * @param http_msg the HTTP message object to populate from parsing
+	 * @param ec error_code contains additional information for parsing errors
 	 *
 	 * @return boost::tribool result of parsing:
 	 *                        false = message has an error,
 	 *                        true = finished parsing HTTP message,
 	 *                        indeterminate = not yet finished parsing HTTP message
 	 */
-	boost::tribool parse(HTTPMessage& http_msg);
+	boost::tribool parse(HTTPMessage& http_msg, boost::system::error_code& ec);
 
 	/**
 	 * attempts to continue parsing despite having missed data (length is known but content is not)
 	 *
 	 * @param http_msg the HTTP message object to populate from parsing
 	 * @param len the length in bytes of the missing data
+	 * @param ec error_code contains additional information for parsing errors
 	 *
 	 * @return boost::tribool result of parsing:
 	 *                        false = message has an error,
 	 *                        true = finished parsing HTTP message,
 	 *                        indeterminate = not yet finished parsing HTTP message
 	 */
-	boost::tribool parseMissingData(HTTPMessage& http_msg, std::size_t len);
+	boost::tribool parseMissingData(HTTPMessage& http_msg, std::size_t len,
+		boost::system::error_code& ec);
 
 	/**
 	 * finishes parsing an HTTP response message
@@ -116,8 +193,7 @@ public:
 	 * should be called if there is no more data to parse, and if the last
 	 * call to the parse() function returned boost::indeterminate
 	 *
-	 * @param http_mhttp://start.fedoraproject.org/sg the HTTP 
-	 *                  message object being parsed
+	 * @param http_msg the HTTP message object being parsed
 	 * @return true if premature EOF, false if message is OK & finished parsing
 	 */
 	inline bool checkPrematureEOF(HTTPMessage& http_msg) {
@@ -129,6 +205,23 @@ public:
 		return false;
 	}
 
+	/**
+	 * controls headers-only parsing (default is disabled; content parsed also)
+	 *
+	 * @param b if true, then the parse() function returns true after headers
+	 */
+	inline void parseHeadersOnly(bool b = true) { m_parse_headers_only = b; }
+
+	/**
+	 * skip parsing all headers and parse payload content only
+	 *
+	 * @param http_msg the HTTP message object being parsed
+	 */
+	inline void skipHeaderParsing(HTTPMessage& http_msg) {
+		boost::system::error_code ec;
+		finishHeaderParsing(http_msg, ec);
+	}
+	
 	/// resets the parser to its initial state
 	inline void reset(void) {
 		m_message_parse_state = PARSE_START;
@@ -189,16 +282,6 @@ public:
 	inline PionLogger getLogger(void) { return m_logger; }
 
 
-	/**
-	 * Determine if the Content-Type header is valid and has type "application/x-www-form-urlencoded",
-	 * and if a charset parameter is found, save the value.
-	 * 
-	 * @param http_request request whose Content-Type header is being analyzed
-	 * 
-	 * @return bool true if header is valid and has type "application/x-www-form-urlencoded"
-	 */
-	static bool contentTypeIsUrlEncoded(HTTPRequest& http_request);
-
 	/**
 	 * parse key-value pairs out of a url-encoded string
 	 * (i.e. this=that&a=value)
@@ -258,6 +341,23 @@ public:
 		return parseURLEncoded(dict, query.c_str(), query.size());
 	}
 
+	/**
+	 * parses an X-Forwarded-For HTTP header, and extracts from it an IP
+	 * address that best matches the client's public IP address (if any are found)
+	 *
+	 * @param header the X-Forwarded-For HTTP header to parse
+	 * @param public_ip the extract IP address, if found
+	 *
+	 * @return bool true if a public IP address was found and extracted
+	 */
+	static bool parseForwardedFor(const std::string& header, std::string& public_ip);
+	
+	/// returns an instance of HTTPParser::ErrorCategory
+	static inline ErrorCategory& getErrorCategory(void) {
+		boost::call_once(HTTPParser::createErrorCategory, m_instance_flag);
+		return *m_error_category_ptr;
+	}
+
 
 protected:
 
@@ -266,13 +366,14 @@ protected:
 	 * available in the read buffer
 	 *
 	 * @param http_msg the HTTP message object to populate from parsing
+	 * @param ec error_code contains additional information for parsing errors
 	 *
 	 * @return boost::tribool result of parsing:
 	 *                        false = message has an error,
 	 *                        true = finished parsing HTTP headers,
 	 *                        indeterminate = not yet finished parsing HTTP headers
 	 */
-	boost::tribool parseHeaders(HTTPMessage& http_msg);
+	boost::tribool parseHeaders(HTTPMessage& http_msg, boost::system::error_code& ec);
 
 	/**
 	 * updates an HTTPMessage object with data obtained from parsing headers
@@ -286,37 +387,43 @@ protected:
 	 * available in the read buffer
 	 *
 	 * @param http_msg the HTTP message object to populate from parsing
+	 * @param ec error_code contains additional information for parsing errors
 	 *
 	 * @return boost::tribool result of parsing:
 	 *                        false = message has an error,
 	 *                        true = finished parsing HTTP message (no content),
 	 *                        indeterminate = payload content is available to be parsed
 	 */
-	boost::tribool finishHeaderParsing(HTTPMessage& http_msg);
+	boost::tribool finishHeaderParsing(HTTPMessage& http_msg,
+		boost::system::error_code& ec);
 
 	/**
 	 * parses a chunked HTTP message-body using bytes available in the read buffer
 	 *
 	 * @param chunk_buffers buffers to be populated from parsing chunked content
+	 * @param ec error_code contains additional information for parsing errors
 	 *
 	 * @return boost::tribool result of parsing:
 	 *                        false = message has an error,
 	 *                        true = finished parsing message,
 	 *                        indeterminate = message is not yet finished
 	 */
-	boost::tribool parseChunks(HTTPMessage::ChunkCache& chunk_buffers);
+	boost::tribool parseChunks(HTTPMessage::ChunkCache& chunk_buffers,
+		boost::system::error_code& ec);
 
 	/**
 	 * consumes payload content in the parser's read buffer 
 	 *
 	 * @param http_msg the HTTP message object to consume content for
+	 * @param ec error_code contains additional information for parsing errors
 	 *  
 	 * @return boost::tribool result of parsing:
 	 *                        false = message has an error,
 	 *                        true = finished parsing message,
 	 *                        indeterminate = message is not yet finished
 	 */
-	boost::tribool consumeContent(HTTPMessage& http_msg);
+	boost::tribool consumeContent(HTTPMessage& http_msg,
+		boost::system::error_code& ec);
 
 	/**
 	 * consume the bytes available in the read buffer, converting them into
@@ -334,6 +441,20 @@ protected:
 	 */
 	static void computeMsgStatus(HTTPMessage& http_msg, bool msg_parsed_ok);
 
+	/**
+	 * sets an error code
+	 *
+	 * @param ec error code variable to define
+	 * @param ev error value to raise
+	 */
+	static inline void setError(boost::system::error_code& ec, ErrorValue ev) {
+		ec = boost::system::error_code(static_cast<int>(ev), getErrorCategory());
+	}
+
+	/// creates the unique HTTPParser ErrorCategory
+	static void createErrorCategory(void);
+
+
 	// misc functions used by the parsing functions
 	inline static bool isChar(int c);
 	inline static bool isControl(int c);
@@ -479,8 +600,17 @@ private:
 	/// maximum length for HTTP payload content
 	std::size_t							m_max_content_length;
 	
+	/// if true, then only HTTP headers will be parsed (no content parsing)
+	bool								m_parse_headers_only;
+
 	/// if true, the raw contents of HTTP headers are stored into m_raw_headers
 	bool								m_save_raw_headers;
+
+	/// points to a single and unique instance of the HTTPParser ErrorCategory
+	static ErrorCategory *				m_error_category_ptr;
+		
+	/// used to ensure thread safety of the HTTPParser ErrorCategory
+	static boost::once_flag				m_instance_flag;
 };
 
 
