@@ -192,6 +192,28 @@ public:
 								  TCPConnectionPtr& tcp_conn,
 								  const std::string& error_msg);
 
+	/**
+	 * used to send responses when a request is forbidden
+	 *
+	 * @param http_request the new HTTP request to handle
+	 * @param tcp_conn the TCP connection that has the new request
+	 * @param error_msg message that explains what went wrong
+	 */
+	static void handleForbiddenRequest(HTTPRequestPtr& http_request,
+									   TCPConnectionPtr& tcp_conn,
+									   const std::string& error_msg);
+
+	/**
+	 * used to send responses when a method is not allowed
+	 *
+	 * @param http_request the new HTTP request to handle
+	 * @param tcp_conn the TCP connection that has the new request
+	 * @param allowed_methods optional comma separated list of allowed methods
+	 */
+	static void handleMethodNotAllowed(HTTPRequestPtr& http_request,
+									   TCPConnectionPtr& tcp_conn,
+									   const std::string& allowed_methods = "");
+
 	/**
 	 * sets the handler object for authentication verification processing
 	 */ 
@@ -214,8 +236,10 @@ protected:
 	 *
 	 * @param http_request the HTTP request to handle
 	 * @param tcp_conn TCP connection containing a new request
+	 * @param ec error_code contains additional information for parsing errors
 	 */
-	void handleRequest(HTTPRequestPtr& http_request, TCPConnectionPtr& tcp_conn);
+	virtual void handleRequest(HTTPRequestPtr& http_request,
+		TCPConnectionPtr& tcp_conn, const boost::system::error_code& ec);
 
 	/**
 	 * searches for the appropriate request handler to use for a given resource
@@ -223,7 +247,7 @@ protected:
 	 * @param resource the name of the resource to search for
 	 * @param request_handler function that can handle requests for this resource
 	 */
-	bool findRequestHandler(const std::string& resource,
+	virtual bool findRequestHandler(const std::string& resource,
 							RequestHandler& request_handler) const;
 
 
