@@ -28,18 +28,25 @@ void HTTPServer::handleConnection(TCPConnectionPtr& tcp_conn)
 {
 	HTTPRequestReaderPtr reader_ptr;
 	reader_ptr = HTTPRequestReader::create(tcp_conn, boost::bind(&HTTPServer::handleRequest,
-										   this, _1, _2));
+										   this, _1, _2, _3));
 	reader_ptr->setMaxContentLength(m_max_content_length);
 	reader_ptr->receive();
 }
 
 void HTTPServer::handleRequest(HTTPRequestPtr& http_request,
-							   TCPConnectionPtr& tcp_conn)
+	TCPConnectionPtr& tcp_conn, const boost::system::error_code& ec)
 {
-	if (! http_request->isValid()) {
-		// the request is invalid or an error occured
-		PION_LOG_INFO(m_logger, "Received an invalid HTTP request");
-		m_bad_request_handler(http_request, tcp_conn);
+	if (ec || ! http_request->isValid()) {
+		tcp_conn->setLifecycle(TCPConnection::LIFECYCLE_CLOSE);	// make sure it will get closed
+		if (tcp_conn->is_open() && (&ec.category() == &HTTPParser::getErrorCategory())) {
+			// HTTP parser error
+			PION_LOG_INFO(m_logger, "Invalid HTTP request (" << ec.message() << ")");
+			m_bad_request_handler(http_request, tcp_conn);
+		} else {
+			// other (IO) error
+			PION_LOG_INFO(m_logger, "Lost connection on port " << getPort());
+			tcp_conn->finish();
+		}
 		return;
 	}
 		
@@ -88,11 +95,6 @@ void HTTPServer::handleRequest(HTTPRequestPtr& http_request,
 			if (http_request->getResource() != http_request->getOriginalResource()) {
 				PION_LOG_DEBUG(m_logger, "Original resource requested was: " << http_request->getOriginalResource());
 			}
-		} catch (HTTPResponseWriter::LostConnectionException& e) {
-			// the connection was lost while or before sending the response
-			PION_LOG_WARN(m_logger, "HTTP request handler: " << e.what());
-			tcp_conn->setLifecycle(TCPConnection::LIFECYCLE_CLOSE);	// make sure it will get closed
-			tcp_conn->finish();
 		} catch (std::bad_alloc&) {
 			// propagate memory errors (FATAL)
 			throw;
@@ -230,6 +232,58 @@ void HTTPServer::handleServerError(HTTPRequestPtr& http_request,
 	writer->send();
 }
 
+void HTTPServer::handleForbiddenRequest(HTTPRequestPtr& http_request,
+										TCPConnectionPtr& tcp_conn,
+										const std::string& error_msg)
+{
+	static const std::string FORBIDDEN_HTML_START =
+		"<html><head>\n"
+		"<title>403 Forbidden</title>\n"
+		"</head><body>\n"
+		"<h1>Forbidden</h1>\n"
+		"<p>User not authorized to access the requested URL ";
+	static const std::string FORBIDDEN_HTML_MIDDLE =
+		"</p><p><strong>\n";
+	static const std::string FORBIDDEN_HTML_FINISH =
+		"</strong></p>\n"
+		"</body></html>\n";
+	HTTPResponseWriterPtr writer(HTTPResponseWriter::create(tcp_conn, *http_request,
+															boost::bind(&TCPConnection::finish, tcp_conn)));
+	writer->getResponse().setStatusCode(HTTPTypes::RESPONSE_CODE_FORBIDDEN);
+	writer->getResponse().setStatusMessage(HTTPTypes::RESPONSE_MESSAGE_FORBIDDEN);
+	writer->writeNoCopy(FORBIDDEN_HTML_START);
+	writer << http_request->getResource();
+	writer->writeNoCopy(FORBIDDEN_HTML_MIDDLE);
+	writer << error_msg;
+	writer->writeNoCopy(FORBIDDEN_HTML_FINISH);
+	writer->send();
+}
+
+void HTTPServer::handleMethodNotAllowed(HTTPRequestPtr& http_request,
+										TCPConnectionPtr& tcp_conn,
+										const std::string& allowed_methods)
+{
+	static const std::string NOT_ALLOWED_HTML_START =
+		"<html><head>\n"
+		"<title>405 Method Not Allowed</title>\n"
+		"</head><body>\n"
+		"<h1>Not Allowed</h1>\n"
+		"<p>The requested method ";
+	static const std::string NOT_ALLOWED_HTML_FINISH =
+		" is not allowed on this server.</p>\n"
+		"</body></html>\n";
+	HTTPResponseWriterPtr writer(HTTPResponseWriter::create(tcp_conn, *http_request,
+															boost::bind(&TCPConnection::finish, tcp_conn)));
+	writer->getResponse().setStatusCode(HTTPTypes::RESPONSE_CODE_METHOD_NOT_ALLOWED);
+	writer->getResponse().setStatusMessage(HTTPTypes::RESPONSE_MESSAGE_METHOD_NOT_ALLOWED);
+	if (! allowed_methods.empty())
+		writer->getResponse().addHeader("Allow", allowed_methods);
+	writer->writeNoCopy(NOT_ALLOWED_HTML_START);
+	writer << http_request->getMethod();
+	writer->writeNoCopy(NOT_ALLOWED_HTML_FINISH);
+	writer->send();
+}
+
 }	// end namespace net
 }	// end namespace pion
 
